function parseTwoslashAssertion(comment, commentIndex, commentLine, sourceText, lineStarts) {
  const matchTwoslash = /^( *)\^\?(.*)$/.exec(comment);
  if (!matchTwoslash) {
    return null;
  }
  const whitespace = matchTwoslash[1];
  const rawPayload = matchTwoslash[2];
  if (rawPayload.length && !rawPayload.startsWith(" ")) {
    return {
      line: commentLine - 1,
      type: "InvalidTwoslash"
    };
  }
  let expected = rawPayload.slice(1);
  if (commentLine === 1) {
    return {
      assertionType: "twoslash",
      expected,
      expectedPrefix: "",
      expectedRange: [-1, -1],
      insertSpace: false,
      position: -1
    };
  }
  const caretIndex = commentIndex + whitespace.length + 2;
  const position = caretIndex - (lineStarts[commentLine - 1] - lineStarts[commentLine - 2]);
  const expectedRange = [
    commentIndex + whitespace.length + 5,
    commentLine < lineStarts.length ? lineStarts[commentLine] - 1 : sourceText.length
  ];
  const expectedPrefix = sourceText.slice(
    lineStarts[commentLine - 1],
    commentIndex + 2 + whitespace.length
  ) + "   ";
  for (let nextLine = commentLine; nextLine < lineStarts.length; nextLine++) {
    const thisLineEnd = nextLine + 1 < lineStarts.length ? lineStarts[nextLine + 1] - 1 : sourceText.length;
    const lineText = sourceText.slice(lineStarts[nextLine], thisLineEnd + 1);
    if (lineText.startsWith(expectedPrefix)) {
      if (nextLine === commentLine) {
        expected += "\n";
      }
      expected += lineText.slice(expectedPrefix.length);
      expectedRange[1] = thisLineEnd;
    } else {
      break;
    }
  }
  let insertSpace = false;
  if (expectedRange[0] > expectedRange[1]) {
    expectedRange[0] = expectedRange[1];
    insertSpace = true;
  }
  return {
    assertionType: "twoslash",
    expected,
    expectedPrefix,
    expectedRange,
    insertSpace,
    position
  };
}
export {
  parseTwoslashAssertion
};
//# sourceMappingURL=parseTwoslashAssertion.mjs.map