import ts from "typescript";
import { getNodeAtPosition } from "../utils/locations.js";
import { lineOfPosition } from "../utils/locations.js";
import {
  getLanguageServiceHost,
  getNodeForExpectType,
  matchModuloWhitespace,
  matchReadonlyArray
} from "../utils/typescript.js";
function getExpectTypeFailures(sourceFile, assertions, program) {
  const checker = program.getTypeChecker();
  const languageService = ts.createLanguageService(
    getLanguageServiceHost(program)
  );
  const { twoSlashAssertions, typeAssertions } = assertions;
  const unmetExpectations = [];
  ts.forEachChild(sourceFile, function iterate(node) {
    const line = lineOfPosition(node.getStart(sourceFile), sourceFile);
    const assertion = typeAssertions.get(line);
    if (assertion !== void 0) {
      const { expected } = assertion;
      let nodeToCheck = node;
      if (node.kind === ts.SyntaxKind.ExpressionStatement) {
        node = node.expression;
      }
      nodeToCheck = getNodeForExpectType(node);
      const type = checker.getTypeAtLocation(nodeToCheck);
      const actual = checker.typeToString(
        type,
        /*enclosingDeclaration*/
        void 0,
        ts.TypeFormatFlags.NoTruncation
      );
      if (!expected || actual !== expected && !matchReadonlyArray(actual, expected)) {
        unmetExpectations.push({ actual, assertion, node });
      }
      typeAssertions.delete(line);
    }
    ts.forEachChild(node, iterate);
  });
  const twoSlashFailureLines = [];
  if (twoSlashAssertions.length) {
    for (const assertion of twoSlashAssertions) {
      const { expected, position } = assertion;
      if (position === -1) {
        twoSlashFailureLines.push(0);
        continue;
      }
      const node = getNodeAtPosition(sourceFile, position);
      if (!node) {
        twoSlashFailureLines.push(
          sourceFile.getLineAndCharacterOfPosition(position).line
        );
        continue;
      }
      const qi = languageService.getQuickInfoAtPosition(
        sourceFile.fileName,
        node.getStart()
      );
      if (!qi?.displayParts) {
        twoSlashFailureLines.push(
          sourceFile.getLineAndCharacterOfPosition(position).line
        );
        continue;
      }
      const actual = qi.displayParts.map((dp) => dp.text).join("");
      if (!matchModuloWhitespace(actual, expected)) {
        unmetExpectations.push({
          actual,
          assertion,
          node
        });
      }
    }
  }
  return {
    unmetExpectations,
    unusedAssertions: [...twoSlashFailureLines, ...typeAssertions.keys()]
  };
}
export {
  getExpectTypeFailures
};
//# sourceMappingURL=getExpectTypeFailures.mjs.map