"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var expect_exports = {};
__export(expect_exports, {
  expect: () => expect
});
module.exports = __toCommonJS(expect_exports);
var import_utils = require("@typescript-eslint/utils");
var import_typescript = __toESM(require("typescript"));
var import_parseAssertions = require("../assertions/parseAssertions.js");
var import_getExpectTypeFailures = require("../failures/getExpectTypeFailures.js");
var import_createRule = require("../utils/createRule.js");
var import_diagnostics = require("../utils/diagnostics.js");
var import_locations = require("../utils/locations.js");
var import_snapshot = require("../utils/snapshot.js");
const messages = {
  ExpectedErrorNotFound: "Expected an error on this line, but found none.",
  FileIsNotIncludedInTSConfig: 'Expected to find a file "{{ fileName }}" present.',
  Multiple$ExpectTypeAssertions: "This line has 2 or more $ExpectType assertions.",
  OrphanAssertion: "Can not match a node to this assertion.",
  SyntaxError: "Syntax Error: {{ message }}",
  TypeSnapshotDoNotMatch: "Expected type from Snapshot to be: {{ expected }}, got: {{ actual }}",
  TypeSnapshotNotFound: "Type Snapshot not found. Please consider running ESLint in FIX mode: eslint --fix",
  TypesDoNotMatch: "Expected type to be: {{ expected }}, got: {{ actual }}"
};
const defaultOptions = {
  disableExpectTypeSnapshotFix: false
};
const expect = (0, import_createRule.createRule)({
  create(context, [options]) {
    const parserServices = import_utils.ESLintUtils.getParserServices(context);
    const { program } = parserServices;
    const fileName = context.filename || context.getFilename();
    const sourceFile = program.getSourceFile(fileName);
    if (!sourceFile) {
      context.report({
        data: {
          fileName
        },
        loc: {
          column: 0,
          line: 1
        },
        messageId: "FileIsNotIncludedInTSConfig"
      });
      return {};
    }
    if (!/\$Expect(?:Type|Error|\?)|\^\?/.test(sourceFile.text)) {
      return {};
    }
    const {
      duplicates,
      errorLines,
      syntaxErrors,
      twoSlashAssertions,
      typeAssertions
    } = (0, import_parseAssertions.parseAssertions)(sourceFile);
    reportDuplicates(context, duplicates);
    reportNotFoundErrors(context, errorLines, program, sourceFile);
    reportSyntaxErrors(context, syntaxErrors);
    const { unmetExpectations, unusedAssertions } = (0, import_getExpectTypeFailures.getExpectTypeFailures)(
      sourceFile,
      { twoSlashAssertions, typeAssertions },
      program
    );
    reportUnmetExpectations(context, options, unmetExpectations, sourceFile);
    reportUnusedAssertions(context, unusedAssertions);
    return {};
  },
  defaultOptions: [defaultOptions],
  meta: {
    docs: {
      description: "Expects type error, type snapshot, or type.",
      requiresTypeChecking: true
    },
    fixable: "code",
    messages,
    schema: [
      {
        additionalProperties: false,
        properties: {
          disableExpectTypeSnapshotFix: {
            type: "boolean"
          }
        },
        type: "object"
      }
    ],
    type: "problem"
  },
  name: "expect"
});
function reportDuplicates(context, duplicates) {
  for (const line of duplicates) {
    context.report({
      loc: {
        column: 0,
        line: line + 1
      },
      messageId: "Multiple$ExpectTypeAssertions"
    });
  }
}
function reportNotFoundErrors(context, errorLines, program, sourceFile) {
  const diagnostics = import_typescript.default.getPreEmitDiagnostics(program, sourceFile);
  const seenDiagnosticsOnLine = new Set(
    diagnostics.filter(import_diagnostics.isDiagnosticWithStart).map((diagnostic) => (0, import_locations.lineOfPosition)(diagnostic.start, sourceFile))
  );
  for (const line of errorLines) {
    if (!seenDiagnosticsOnLine.has(line)) {
      context.report({
        loc: {
          column: 0,
          line: line + 1
        },
        messageId: "ExpectedErrorNotFound"
      });
    }
  }
}
function reportSyntaxErrors(context, syntaxErrors) {
  for (const { line, type } of syntaxErrors) {
    context.report({
      data: {
        message: type === "MissingExpectType" ? '$ExpectType requires type argument (e.g. // $ExpectType "string")' : type === "MissingSnapshotName" ? "$ExpectTypeSnapshot requires snapshot name argument (e.g. // $ExpectTypeSnapshot MainComponentAPI)" : 'Invalid twoslash assertion; make sure there is a space after the "^?".'
      },
      loc: {
        column: 0,
        line: line + 1
      },
      messageId: "SyntaxError"
    });
  }
}
function reportUnmetExpectations(context, options, unmetExpectations, sourceFile) {
  for (const { actual, assertion, node } of unmetExpectations) {
    const templateDescriptor = {
      data: {
        actual,
        expected: assertion.expected
      },
      loc: (0, import_locations.locForTSNode)(sourceFile, node)
    };
    if (assertion.assertionType === "snapshot") {
      const { snapshotName } = assertion;
      const start = node.getStart();
      context.report({
        ...templateDescriptor,
        // ESLint doesn't indicate whether it's in --fix mode, and fixers run immediately.
        // We don't have a way to delay file updates natively in ESLint.
        // Instead, we use this inaccurate heuristic to know if we're in CLI --fix...
        // See: https://github.com/JoshuaKGoldberg/eslint-plugin-expect-type/issues/14
        fix: process.argv.includes("--fix") ? () => {
          return {
            range: [start, start],
            get text() {
              if (!options.disableExpectTypeSnapshotFix) {
                (0, import_snapshot.updateTypeSnapshot)(
                  // TODO: Once ESLint <8 support is removed, soon
                  // eslint-disable-next-line deprecation/deprecation
                  context.filename || context.getFilename(),
                  snapshotName,
                  actual
                );
              }
              return "";
            }
          };
        } : void 0,
        messageId: typeof assertion.expected === "undefined" ? "TypeSnapshotNotFound" : "TypeSnapshotDoNotMatch"
      });
    } else {
      context.report({
        ...templateDescriptor,
        messageId: "TypesDoNotMatch",
        ...assertion.assertionType === "twoslash" ? {
          fix: () => {
            const { expectedPrefix, expectedRange, insertSpace } = assertion;
            return {
              range: expectedRange,
              text: (insertSpace ? " " : "") + actual.split("\n").map((line, i) => i > 0 ? expectedPrefix + line : line).join("\n")
            };
          }
        } : {}
      });
    }
  }
}
function reportUnusedAssertions(context, unusedAssertions) {
  for (const line of unusedAssertions) {
    context.report({
      loc: {
        column: 0,
        line: line + 1
      },
      messageId: "OrphanAssertion"
    });
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  expect
});
//# sourceMappingURL=expect.js.map