import * as fse from "fs-extra";
import { basename, dirname, resolve } from "path";
const { ensureFileSync, readJsonSync, writeJsonSync } = fse;
const getTypeSnapshot = (filename, snapshotName) => {
  const snapshotPath = getSnapshotPath(filename);
  const json = readJsonSync(snapshotPath, { throws: false });
  if (!json) {
    return;
  }
  return json[snapshotName];
};
const updateTypeSnapshot = (filename, snapshotName, actualType) => {
  const snapshotPath = getSnapshotPath(filename);
  ensureFileSync(snapshotPath);
  const json = readJsonSync(snapshotPath, { throws: false }) ?? {};
  json[snapshotName] = actualType;
  writeJsonSync(snapshotPath, json, { spaces: 2 });
};
function getSnapshotPath(filename) {
  const directory = dirname(filename);
  return resolve(
    directory,
    "__type-snapshots__",
    `${basename(filename)}.snap.json`
  );
}
export {
  getTypeSnapshot,
  updateTypeSnapshot
};
//# sourceMappingURL=snapshot.mjs.map