import * as ts from "typescript";
function getLanguageServiceHost(program) {
  return {
    getCompilationSettings: () => program.getCompilerOptions(),
    getCurrentDirectory: () => program.getCurrentDirectory(),
    getDefaultLibFileName: (options) => ts.getDefaultLibFilePath(options),
    getScriptFileNames: () => program.getSourceFiles().map((sourceFile) => sourceFile.fileName),
    getScriptSnapshot: (name) => ts.ScriptSnapshot.fromString(program.getSourceFile(name)?.text ?? ""),
    getScriptVersion: () => "1",
    // NB: We can't check `program` for files, it won't contain valid files like package.json
    /* eslint-disable @typescript-eslint/unbound-method */
    directoryExists: ts.sys.directoryExists,
    fileExists: ts.sys.fileExists,
    getDirectories: ts.sys.getDirectories,
    readDirectory: ts.sys.readDirectory,
    readFile: ts.sys.readFile
    /* eslint-enable @typescript-eslint/unbound-method */
  };
}
function getNodeForExpectType(node) {
  if (ts.isVariableStatement(node)) {
    const {
      declarationList: { declarations }
    } = node;
    if (declarations.length === 1) {
      const { initializer } = declarations[0];
      if (initializer) {
        return initializer;
      }
    }
  }
  return node;
}
function matchReadonlyArray(actual, expected) {
  if (!(/\breadonly\b/.test(actual) && /\bReadonlyArray\b/.test(expected))) {
    return false;
  }
  const readonlyArrayRegExp = /\bReadonlyArray</y;
  const readonlyModifierRegExp = /\breadonly /y;
  let expectedPos = 0;
  let actualPos = 0;
  let depth = 0;
  while (expectedPos < expected.length && actualPos < actual.length) {
    const expectedChar = expected.charAt(expectedPos);
    const actualChar = actual.charAt(actualPos);
    if (expectedChar === actualChar) {
      expectedPos++;
      actualPos++;
      continue;
    }
    if (depth > 0 && expectedChar === ">" && actualChar === "[" && actualPos < actual.length - 1 && actual.charAt(actualPos + 1) === "]") {
      depth--;
      expectedPos++;
      actualPos += 2;
      continue;
    }
    readonlyArrayRegExp.lastIndex = expectedPos;
    readonlyModifierRegExp.lastIndex = actualPos;
    if (readonlyArrayRegExp.test(expected) && readonlyModifierRegExp.test(actual)) {
      depth++;
      expectedPos += 14;
      actualPos += 9;
      continue;
    }
    return false;
  }
  return true;
}
function matchModuloWhitespace(actual, expected) {
  const normActual = actual.replace(/[\n\r ]+/g, " ").trim();
  const normExpected = expected.replace(/[\n\r ]+/g, " ").trim();
  return normActual === normExpected;
}
export {
  getLanguageServiceHost,
  getNodeForExpectType,
  matchModuloWhitespace,
  matchReadonlyArray
};
//# sourceMappingURL=typescript.mjs.map